#region License
// Copyright (c) 2007 James Newton-King
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without
// restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following
// conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using NUnit.Framework;
using Utilities.Miscellaneous;
using Utilities.Testing;

namespace Utilities.Test.Miscellaneous
{
  public class SingletonProviderFixture : TestFixtureBase
  {
    [Test]
    public void ArraySingletonTest()
    {
      int counter = 0;

      SingletonProvider<int[]> singletonProvider = new SingletonProvider<int[]>(delegate
      {
        return new int[] { counter++ };
      });

      int[] first = singletonProvider.Value;

      Assert.AreEqual(counter, 1);

      Assert.AreEqual(0, first[0]);

      int[] second = singletonProvider.Value;

      // counter shouldn't have changed
      Assert.AreEqual(counter, 1);

      Assert.AreEqual(0, second[0]);

      // objects should be the same
      Assert.AreEqual(first, second);
    }

    [Test]
    [ExpectedException(typeof(ArgumentNullException), ExpectedMessage = @"Value cannot be null.
Parameter name: creator")]
    public void NullCreatorDelegateTest()
    {
      new SingletonProvider<object>(null);
    }

    [Test]
    [ExpectedException(typeof(Exception), ExpectedMessage = "Creator delegate cannot return null.")]
    public void CreatorDelegateReturnNullTest()
    {
      SingletonProvider<object> singletonProvider = new SingletonProvider<object>(delegate
      {
        return null;
      });

      Assert.IsNotNull(singletonProvider.Value);
    }
  }
}
